'use strict';Object.defineProperty(exports, "__esModule", { value: true });
var _querystring = require('./lookups/querystring');var _querystring2 = _interopRequireDefault(_querystring);
var _path = require('./lookups/path');var _path2 = _interopRequireDefault(_path);
var _cookie = require('./lookups/cookie');var _cookie2 = _interopRequireDefault(_cookie);
var _header = require('./lookups/header');var _header2 = _interopRequireDefault(_header);
var _session = require('./lookups/session');var _session2 = _interopRequireDefault(_session);function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };}const assert = require('assert');

function getDefaultsOpt() {
    return {
        order: ['querystring', 'path', 'cookie', 'header', 'session'],

        lookupQuerystring: 'lng',

        lookupParam: 'lng', // for route like: 'path1/:lng/result'
        lookupFromPathIndex: 0,

        lookupCookie: 'i18next',
        // cookieExpirationDate: new Date(),
        // cookieDomain: '',

        lookupSession: 'lng',

        // cache user language
        caches: false //['cookie']
    };
}

function defaults(obj) {
    Array.from(arguments).slice(1).forEach(source => {
        if (source) {
            for (var prop in source) {
                if (obj[prop] === undefined) obj[prop] = source[prop];
            }
        }
    });
    return obj;
}

class LanguageDetector {
    constructor(services, options = {}, i18nextOptions = {}) {
        this.type = 'languageDetector';
        this.async = true;
        this.detectors = {};

        this.init(services, options, i18nextOptions);
    }

    init(services, options = {}, i18nextOptions = {}) {
        this.services = services;
        this.options = defaults(options, this.options || {}, getDefaultsOpt());
        this.i18nextOptions = i18nextOptions;
        try {
            this.addDetector(_querystring2.default);
            this.addDetector(_path2.default);
            this.addDetector(_cookie2.default);
            this.addDetector(_header2.default);
            this.addDetector(_session2.default);
        } catch (e) {
            console.error(e.message);
        }
    }

    addDetector(detector) {
        assert.ok(detector.lookup, `detector must contains a lookup method`);
        this.detectors[detector.name] = detector;
    }

    async detect(ctx) {
        if (arguments.length < 1) {
            return;
        }

        let found;
        for (let detectorName of this.options.order) {
            if (found || !this.detectors[detectorName] || !this.detectors[detectorName].lookup) {
                return;
            }

            let detections;
            if (this.detectors[detectorName].async) {
                detections = await this.detectors[detectorName].lookup(ctx, this.options);
            } else {
                detections = this.detectors[detectorName].lookup(ctx, this.options);
            }

            if (!detections) return;
            if (typeof detections === 'string') {
                detections = [detections];
            }

            detections.forEach(lng => {
                if (found) return;

                let cleanedLng = this.services.languageUtils.formatLanguageCode(lng);

                if (this.services.languageUtils.isWhitelisted(cleanedLng)) {
                    found = cleanedLng;
                    ctx.i18nextLookupName = detectorName;
                };
            });
        }

        return found || this.i18nextOptions.fallbackLng && this.i18nextOptions.fallbackLng[0];
    }

    cacheUserLanguage(ctx, lng, caches) {
        if (arguments.length < 2) {
            return;
        }
        if (!caches) {
            caches = this.options.caches;
        }
        if (!caches) {
            return;
        }
        caches.forEach(cacheName => {
            let detector = this.detectors[cacheName];
            if (detector && detector.cacheUserLanguage) {
                detector.cacheUserLanguage(ctx, lng, this.options);
            }
        });
    }}exports.default =


LanguageDetector;